import ctypes
import os
import time

from subsearch.data import __version__, __video__
from subsearch.gui import widget_menu
from subsearch.providers import opensubtitles, subscene
from subsearch.utils import (
    current_user,
    file_manager,
    log,
    raw_config,
    raw_registry,
    string_parser,
)


class BaseInitializer:
    def __init__(self):
        self.current_language = raw_config.get_config_key("current_language")
        self.languages = raw_config.get_config_key("languages")
        self.percentage = raw_config.get_config_key("percentage")
        self.rename_best_match = raw_config.get_config_key("rename_best_match")
        self.show_download_window = raw_config.get_config_key("show_download_window")
        self.show_terminal = raw_config.get_config_key("show_terminal")
        subtitle_type = raw_config.get_config_key("subtitle_type")
        self.hearing_impaired = subtitle_type["hearing_impaired"]
        self.non_hearing_impaired = subtitle_type["non_hearing_impaired"]
        providers = raw_config.get_config_key("providers")
        self.pro_subscene = providers["subscene_site"]
        self.pro_opensubtitles_rss = providers["opensubtitles_rss"]
        self.pro_opensubtitles_hash = providers["opensubtitles_hash"]
        self.file_exist = True if __video__.name is not None else False
        self.file_hash = file_manager.get_hash(__video__.path)
        self.user_parameters = raw_config.UserParameters(
            current_language=self.current_language,
            languages=self.languages,
            hearing_impaired=self.hearing_impaired,
            non_hearing_impaired=self.non_hearing_impaired,
            percentage=self.percentage,
            show_download_window=self.show_download_window,
        )
        self.opensubtitles_hash_results = None
        self.opensubtitles_rss_results = None
        self.subscene_results = None
        self.opensubtitles_sorted_list = []
        self.subscene_sorted_list = []
        self.combined_sorted_list = []
        self.opensubtitles_hash_downloads = 0
        self.opensubtitles_rss_downloads = 0
        self.subscene_downloads = 0
        self.ran_download_tab = False
        if self.file_exist:
            self.file_hash = file_manager.get_hash(__video__.path)
            self.parameters = string_parser.get_parameters(__video__.name, self.file_hash, self.user_parameters)
            log.parameters(self.parameters, self.user_parameters)


class Steps(BaseInitializer):
    def __init__(self):
        self.start = time.perf_counter()
        BaseInitializer.__init__(self)
        ctypes.windll.kernel32.SetConsoleTitleW(f"subsearch - {__version__}")
        if current_user.got_key() is False:
            raw_config.set_default_json()
            raw_registry.add_context_menu()
        if __video__.tmp_directory is not None:
            if not os.path.exists(__video__.tmp_directory):
                os.mkdir(__video__.tmp_directory)
            if not os.path.exists(__video__.subs_directory):
                os.mkdir(__video__.subs_directory)
        if self.file_exist is False:
            widget_menu.open_tab("search")
            return None

        if " " in __video__.name:
            log.output("[Warning: Filename contains spaces]")

    def _provider_opensubtitles(self):
        if self.file_exist is False:
            return None
        if self.languages[self.current_language] == "N/A":
            log.output("\n[Searching on opensubtitles]")
            log.output(f"{self.current_language} not supported on opensubtitles\n")
            return None

        _opensubtitles = opensubtitles.OpenSubtitles(self.parameters, self.user_parameters)
        if self.pro_opensubtitles_hash and self.file_hash is not None:
            log.output("\n[Searching on opensubtitles - hash]")
            self.opensubtitles_hash = _opensubtitles.parse_hash()
        if self.pro_opensubtitles_rss:
            log.output("\n[Searching on opensubtitles - rss]")
            self.opensubtitles_rss = _opensubtitles.parse_rss()

        self.opensubtitles_sorted_list = _opensubtitles.sorted_list()

    def _provider_subscene(self):
        if self.file_exist is False:
            return None
        if self.pro_subscene is False:
            return None
        log.output("\n[Searching on subscene - title]")
        _subscene = subscene.Subscene(self.parameters, self.user_parameters)
        self.subscene = _subscene.parse()
        self.subscene_sorted_list = _subscene.sorted_list()

    def _download_files(self):
        if self.file_exist is False:
            return None
        if self.pro_opensubtitles_hash and self.opensubtitles_hash is not None:
            log.output("\n[Downloading from opensubtitles - hash]")
            for item in self.opensubtitles_hash:
                self.opensubtitles_hash_downloads = file_manager.download_subtitle(item)
                log.output("Done with tasks")
                log.output("\n")
        if self.pro_opensubtitles_rss and self.opensubtitles_rss is not None:
            log.output("\n[Downloading from opensubtitles - rss]")
            for item in self.opensubtitles_rss:
                self.opensubtitles_rss_downloads = file_manager.download_subtitle(item)
                log.output("Done with tasks")
        if self.pro_subscene and self.subscene is not None:
            log.output("\n[Downloading from subscene]")
            for item in self.subscene:
                self.subscene_downloads = file_manager.download_subtitle(item)
                log.output("Done with tasks")

    def _not_downloaded(self):
        if self.file_exist is False:
            return None
        total_dls = self.opensubtitles_hash_downloads + self.opensubtitles_rss_downloads + self.subscene_downloads
        if self.show_download_window and total_dls == 0:
            self.combined_sorted_list = list(self.opensubtitles_sorted_list)
            self.combined_sorted_list.extend(x for x in self.subscene_sorted_list if x not in self.combined_sorted_list)
            self.combined_sorted_list.sort(key=lambda x: x[0], reverse=True)
        if len(self.combined_sorted_list) > 0:
            file_manager.write_not_downloaded_tmp(__video__.tmp_directory, self.combined_sorted_list)
            widget_menu.open_tab("download")
            self.ran_download_tab = True

    def _extract_zip_files(self):
        if self.file_exist is False:
            return None
        if self.ran_download_tab:
            return None
        log.output("\n[Proccessing downloads]")
        file_manager.extract_files(__video__.tmp_directory, __video__.subs_directory, ".zip")

    def _clean_up(self):
        if self.file_exist is False:
            return None
        if self.rename_best_match:
            file_manager.rename_best_match(f"{self.parameters.release}.srt", __video__.directory, ".srt")
        log.output("\n[Cleaning up]")
        file_manager.clean_up_files(__video__.subs_directory, "nfo")
        file_manager.del_directory(__video__.tmp_directory)
        log.output("Done with tasks")

    def _pre_exit(self):
        elapsed = time.perf_counter() - self.start
        log.output(f"\nFinished in {elapsed} seconds")

        if self.show_terminal and current_user.check_is_exe() is False:
            try:
                input("Ctrl + c or Enter to exit")
            except KeyboardInterrupt:
                pass



import os
import re
import tkinter as tk
from tkinter import ttk

from subsearch.data import __video__
from subsearch.gui import tk_data
from subsearch.providers import subscene
from subsearch.providers.generic import DownloadData
from subsearch.utils import file_manager, log

TKWINDOW = tk_data.Window()
TKCOLOR = tk_data.Color()
TKFONT = tk_data.Font()
# file with subtitles and corresponding dl links
def read_tmp_file():
    if __video__.tmp_directory == None:
        return None

    file = os.path.join(__video__.tmp_directory, "download_data.tmp")
    with open(file, "r") as f:
        return [line.strip() for line in f]


# download said subtitle to the folder with the video file in it
class DownloadList(tk.Frame):
    def __init__(self, parent, con_x, con_y):
        tk.Frame.__init__(self, parent)
        self.configure(bg=TKCOLOR.dark_grey, width=con_x, height=con_y)
        # listbox for the subtitles
        self.subscene_scrape = subscene.SubsceneScrape()
        self.extent = 0
        self.sublist = read_tmp_file()
        self.scrollbar = ttk.Scrollbar(self, orient="vertical", style="Vertical.TScrollbar")
        self.sub_listbox = tk.Listbox(
            self,
            height=con_y,
            bg=TKCOLOR.dark_grey,
            fg=TKCOLOR.light_grey,
            font=TKFONT.cas8b,
            bd=0,
            border=0,
            borderwidth=0,
            highlightthickness=0,
            activestyle=None,
            yscrollcommand=self.scrollbar.set,
        )
        hsx, hsy = self.scrollbar.winfo_reqwidth(), self.scrollbar.winfo_reqheight()
        self.sub_listbox.place(
            height=con_y - 36,
            width=con_x - hsx - 20,
            x=18,
            rely=0.5,
            bordermode="inside",
            anchor="w",
        )
        if self.sublist is not None:
            self.fill_listbox()
        self.scrollbar.place(x=con_x - 17, y=0, bordermode="inside", height=con_y)
        self.scrollbar.config(command=self.sub_listbox.yview)
        self.scrollbar.lift()

    def fill_listbox(self):
        dicts_names = {}
        dicts_urls = {}
        log.output("\n[Proccessing downloads]")
        # fil list box with all available subtitles that were found and not downloaded
        for enu, item in enumerate(self.sublist):
            x = item.split(" ")
            match = f"{x[0]} {x[1]}"
            name = x[2]
            url = x[-1]
            txt = f"{match} {name}"
            self.sub_listbox.insert(tk.END, f"{match} {name}\n")
            self.sub_listbox.bind("<ButtonPress-1>", self.mouse_b1_press)
            dicts_names[enu] = txt
            dicts_urls[enu] = url
            self.dicts_urls = dicts_urls
            self.dicts_names = dicts_names

    def mouse_b1_press(self, event):
        self.sub_listbox.bind("<<ListboxSelect>>", self.download_button)

    def mouse_b1_release(self, event):
        self.sub_listbox.bind("<ButtonPress-1>", self.mouse_b1_press)

    def download_button(self, event):
        self.sub_listbox.unbind("<<ListboxSelect>>")
        self.sub_listbox.bind("<ButtonRelease-1>", self.mouse_b1_release)
        selection = str(self.sub_listbox.curselection())
        item_num = re.findall("(\d+)", selection)[0]
        self.sub_listbox.delete(int(item_num))
        self.sub_listbox.insert(int(item_num), f"» DOWNLOADING «")
        for (number, _url), (_name) in zip(self.dicts_urls.items(), self.dicts_names.values()):
            if number != int(item_num):
                continue
            self.sub_listbox.itemconfig(int(number), {"fg": TKCOLOR.blue})
            dl_url = self.subscene_scrape.download_url(_url)
            _name = "".join(re.findall("[^\/\\\:\?\<\>\|\*]*", _name))
            path = f"{__video__.tmp_directory}\\__subsearch__{item_num}.zip"
            item = DownloadData(name=_name, file_path=path, url=dl_url, idx_num=1, idx_lenght=1)
            file_manager.download_subtitle(item)
            file_manager.extract_files(__video__.tmp_directory, __video__.subs_directory, ".zip")
            break
        self.sub_listbox.delete(int(item_num))
        self.sub_listbox.insert(int(item_num), f"✔ {_name}")
        self.sub_listbox.itemconfig(int(item_num), {"fg": TKCOLOR.green})



import tkinter as tk

from subsearch import gui
from subsearch.gui import (
    base_root,
    tab_download,
    tab_language,
    tab_search,
    tab_subsearch,
    tk_data,
    tk_tools,
)
from subsearch.utils import current_user

TKWINDOW = tk_data.Window()
TKCOLOR = tk_data.Color()
TKFONT = tk_data.Font()


class Footer(tk.Frame):
    def __init__(self, parent, active_tab: str, language_tab, search_tab, settings_tab, download_tab):
        tk.Frame.__init__(self, parent)
        self.configure(bg=TKCOLOR.mid_grey_black, width=TKWINDOW.width - 4, height=80)
        self.language_tab = language_tab
        self.search_tab = search_tab
        self.settings_tab = settings_tab
        self.download_tab = download_tab
        self.language = tk.Canvas(
            self,
            width=54,
            height=54,
            bg=TKCOLOR.mid_grey_black,
            highlightthickness=0,
        )
        self.search = tk.Canvas(
            self,
            width=54,
            height=54,
            bg=TKCOLOR.mid_grey_black,
            highlightthickness=0,
        )
        self.settings = tk.Canvas(
            self,
            width=54,
            height=54,
            bg=TKCOLOR.mid_grey_black,
            highlightthickness=0,
        )
        self.download = tk.Canvas(
            self,
            width=54,
            height=54,
            bg=TKCOLOR.mid_grey_black,
            highlightthickness=0,
        )
        self.language.place(relx=0.2, rely=0.5, anchor="center")
        self.search.place(relx=0.4, rely=0.5, anchor="center")
        self.settings.place(relx=0.6, rely=0.5, anchor="center")
        self.download.place(relx=0.8, rely=0.5, anchor="center")

        self.language.bind("<Enter>", self.enter_tab)
        self.search.bind("<Enter>", self.enter_tab)
        self.settings.bind("<Enter>", self.enter_tab)
        self.download.bind("<Enter>", self.enter_tab)

        self.language.bind("<Leave>", self.leave_tab)
        self.search.bind("<Leave>", self.leave_tab)
        self.settings.bind("<Leave>", self.leave_tab)
        self.download.bind("<Leave>", self.leave_tab)
        tk_tools.asset_tab(self.language, "language", "rest")
        tk_tools.asset_tab(self.search, "search", "rest")
        tk_tools.asset_tab(self.settings, "settings", "rest")
        tk_tools.asset_tab(self.download, "download", "rest")
        tk_tools.set_default_grid_size(self)
        self.active_tab = active_tab
        self.activate_tabs()

    def activate_tabs(self):
        if self.active_tab == "language":
            self.language_tab.place(relx=0.5, rely=0.5, anchor="center")
            tk_tools.asset_tab(self.language, "language", "press")
        elif self.active_tab == "search":
            self.search_tab.place(relx=0.5, rely=0.5, anchor="center")
            tk_tools.asset_tab(self.search, "search", "press")
        elif self.active_tab == "settings":
            self.settings_tab.place(relx=0.5, rely=0.5, anchor="center")
            tk_tools.asset_tab(self.settings, "settings", "press")
        elif self.active_tab == "download":
            self.download_tab.place(relx=0.5, rely=0.5, anchor="center")
            tk_tools.asset_tab(self.download, "download", "press")

    def release_tab(self, event):
        if event.widget == self.language:
            self.active_tab = "language"
        if event.widget == self.search:
            self.active_tab = "search"
        if event.widget == self.settings:
            self.active_tab = "settings"
        if event.widget == self.download:
            self.active_tab = "download"
        self.activate_tabs()
        self.deactivate_tabs()

    def press_tab(self, event):
        if event.widget == self.language:
            self.language.bind("<ButtonRelease>", self.release_tab)
            tk_tools.asset_tab(self.language, "language", "press", y=20)
        if event.widget == self.search:
            self.search.bind("<ButtonRelease>", self.release_tab)
            tk_tools.asset_tab(self.search, "search", "press", y=20)
        if event.widget == self.settings:
            self.settings.bind("<ButtonRelease>", self.release_tab)
            tk_tools.asset_tab(self.settings, "settings", "press", y=20)
        if event.widget == self.download:
            self.download.bind("<ButtonRelease>", self.release_tab)
            tk_tools.asset_tab(self.download, "download", "press", y=20)

    def deactivate_tabs(self):
        if self.active_tab != "language":
            self.language_tab.place(relx=1, rely=0.5, anchor="nw")
            tk_tools.asset_tab(self.language, "language", "rest")
        if self.active_tab != "search":
            self.search_tab.place(relx=1, rely=0.5, anchor="nw")
            tk_tools.asset_tab(self.search, "search", "rest")
        if self.active_tab != "settings":
            self.settings_tab.place(relx=1, rely=0.5, anchor="nw")
            tk_tools.asset_tab(self.settings, "settings", "rest")
        if self.active_tab != "download":
            self.download_tab.place(relx=1, rely=0.5, anchor="nw")
            tk_tools.asset_tab(self.download, "download", "rest")

    def enter_tab(self, event):
        if event.widget == self.language:
            self.language.bind("<ButtonPress>", self.press_tab)
            tk_tools.asset_tab(self.language, "language", "hover", y=25)
        if event.widget == self.search:
            self.search.bind("<ButtonPress>", self.press_tab)
            tk_tools.asset_tab(self.search, "search", "hover", y=25)
        if event.widget == self.settings:
            self.settings.bind("<ButtonPress>", self.press_tab)
            tk_tools.asset_tab(self.settings, "settings", "hover", y=25)
        if event.widget == self.download:
            self.download.bind("<ButtonPress>", self.press_tab)
            tk_tools.asset_tab(self.download, "download", "hover", y=25)

    def leave_tab(self, event):
        if event.widget == self.language:
            self.language.unbind("<ButtonPress>")
        if event.widget == self.search:
            self.search.unbind("<ButtonPress>")
        if event.widget == self.settings:
            self.settings.unbind("<ButtonPress>")
        if event.widget == self.download:
            self.download.unbind("<ButtonPress>")
        self.activate_tabs()
        self.deactivate_tabs()


class TabLanguage(tk.Frame):
    def __init__(self, parent, content_posx, content_posy):
        tk.Frame.__init__(self, parent)
        self.configure(bg=TKCOLOR.dark_grey)
        tab_language.SelectLanguage(self).pack(anchor="center", expand=True)


class TabSearch(tk.Frame):
    def __init__(self, parent, content_posx, content_posy):
        tk.Frame.__init__(self, parent)
        self.configure(bg=TKCOLOR.dark_grey)
        tab_search.Providers(self).pack(anchor="center")
        tk.Frame(self, height=80, bg=TKCOLOR.dark_grey).pack(anchor="center", expand=True)
        tab_search.SubtitleType(self).pack(anchor="center")
        tab_search.SearchThreshold(self).pack(anchor="center")
        tab_search.RenameBestMatch(self).pack(anchor="center")


class TabSettings(tk.Frame):
    def __init__(self, parent, content_posx, content_posy):
        tk.Frame.__init__(self, parent)
        self.configure(bg=TKCOLOR.dark_grey)
        tab_subsearch.FileExtensions(self).pack(anchor="center")
        tk.Frame(self, height=80, bg=TKCOLOR.dark_grey).pack(anchor="center", expand=True)
        tab_subsearch.ShowContextMenu(self).pack(anchor="center")
        tab_subsearch.ShowContextMenuIcon(self).pack(anchor="center")
        tab_subsearch.ShowDownloadWindow(self).pack(anchor="center")
        if current_user.check_is_exe() is False:
            tab_subsearch.ShowTerminalOnSearch(self).pack(anchor="center")
        tk.Frame(self, height=20, bg=TKCOLOR.dark_grey).pack(anchor="center", expand=True)
        tab_subsearch.CheckForUpdates(self).pack(anchor="center")


class TabDownload(tk.Frame):
    def __init__(self, parent, content_posx, content_posy):
        tk.Frame.__init__(self, parent)
        self.configure(bg=TKCOLOR.dark_grey)
        tab_download.DownloadList(self, content_posx, content_posy).pack(anchor="center")


def open_tab(active_tab: str):
    root = base_root.main()
    gui.set_theme("dark")
    content = tk.Frame(root, bg=TKCOLOR.dark_grey, width=TKWINDOW.width - 4, height=TKWINDOW.height - 118)
    content.place(x=2, y=37)
    content_posx, content_posy = content.winfo_reqwidth(), content.winfo_reqheight()

    footer = Footer(
        root,
        active_tab.lower(),
        TabLanguage(content, content_posx, content_posy),
        TabSearch(content, content_posx, content_posy),
        TabSettings(content, content_posx, content_posy),
        TabDownload(content, content_posx, content_posy),
    )
    footer.place(x=2, y=TKWINDOW.height - 82)
    root.mainloop()



import tkinter as tk

from subsearch.data import __icon__
from subsearch.gui import tk_data, tk_tools
from subsearch.utils import current_user, raw_config, raw_registry

TKWINDOW = tk_data.Window()
TKCOLOR = tk_data.Color()


def check_state(event):
    if hidden_root.state() == "iconic":
        root.wm_withdraw()
    if hidden_root.state() != "iconic":
        root.deiconify()


def main():
    global root, hidden_root
    if current_user.got_key() is False:
        raw_config.set_default_json()
        raw_registry.add_context_menu()
    hidden_root = tk.Tk(className=f"subsearch")
    hidden_root.configure(background=TKCOLOR.black)
    hidden_root.iconbitmap(__icon__)
    hidden_root.geometry(tk_tools.WindowPosition.set(hidden_root))
    hidden_root.attributes("-alpha", 0)
    hidden_root.resizable(False, False)
    root = tk.Toplevel(hidden_root)
    root.geometry(tk_tools.WindowPosition.set(root))
    root.overrideredirect(1)
    tk_tools.CustomBorder(root).place(relx=0.5, rely=0.5, anchor="center")
    tk.Frame(root, height=40, bg=TKCOLOR.dark_grey).pack(anchor="center", expand=True)
    tk_tools.TitleBar(root, hidden_root).place(x=TKWINDOW.width - 2, y=2, bordermode="inside", anchor="ne")
    hidden_root.bind("<FocusOut>", check_state)
    hidden_root.bind("<FocusIn>", check_state)
    return root



from .__set_paths__ import (
    __data__,
    __gui__,
    __home__,
    __icon__,
    __providers__,
    __tabs__,
    __titlebar__,
    __utils__,
)
from .__set_video__ import get_video_file_data
from .__version__ import __version__

__video__ = get_video_file_data()
